'''
data_process:

submodule of TRXASprefitpack for data processing

:copyright: 2021 by pistack (Junho Lee).
:license: LGPL3.
'''

from .data_process import automate_scaling, corr_a_method

__all__ = ['automate_scaling', 'corr_a_method']
