'''
TRXASprefitpack: 
pre fitting package for TRXAS which aims for the first order dynamics
TRXAS stands for Time Resolved X-ray Absorption Spectroscopy

:copyright: 2021 by pistack (Junho Lee)
:license: LGPL3.
'''

from .thy import *
from .data_process import *
from .mathfun import *
from .doc import *
